@@ -2,6 +2,10 @@
 #define ID3LIB_MINOR 0
 #define ID3LIB_PATCH 0
 
+#define LIBFLAC_MAJOR 0
+#define LIBFLAC_MINOR 0
+#define LIBFLAC_PATCH 0
+
 #undef PACKAGE
 #undef VERSION
 #undef HAVE_LIBSM
@@ -11,5 +15,6 @@
 #undef HAVE_STPCPY
 #undef ENABLE_NLS
 #undef ENABLE_OGG
+#undef ENABLE_SYSTEM_LIBFLAC
 #undef HAVE_ICONV_OPEN
 #undef HAVE_LANGINFO_CODESET
