@@ -1,5 +1,6 @@
 #!/bin/sh
-gettextize
+gettextize -c
+libtoolize -c
 aclocal
 autoconf
 autoheader
