@@ -22,60 +22,47 @@
 #ifndef __BROWSER_H__
 #define __BROWSER_H__
 
-
 #include "et_core.h"
 
 /****************
  * Declarations *
  ****************/
 
-/*
- * Data attached to each row of the tree
- */
-typedef struct _DirNode DirNode;
-struct _DirNode
-{
-    gboolean scanned;
-    gchar   *path;
-};
-
-
-/*
- * Data attached to each row of the list of files
- */
-typedef struct _FileRow FileRow;
-struct _FileRow
-{
-    ET_File *ETFile;    // Pointer to the corresponding ETFile structure
-    gboolean colour_bg; // Set to TRUE to colour the row light blue
-};
-
-
 /*
  * Data attached to each row of the artist list
  */
+#if 0
 typedef struct _ArtistRow ArtistRow;
 struct _ArtistRow
 {
     GList *AlbumList; // It's a list of AlbumList items...
 };
-
+#endif
 
 /*
  * Data attached to each row of the artist list
  */
+#if 0
 typedef struct _AlbumRow AlbumRow;
 struct _AlbumRow
 {
     GList *ETFileList; // It's a list of ETFile items...
 };
+#endif
 
+/*
+ * To number columns of ComboBox
+ */
+enum {
+    MISC_COMBO_TEXT, // = 0 (First column)
+    MISC_COMBO_COUNT // = 1 (Number of columns in ComboBox)
+};
 
-GtkWidget *CheckMenuItemBrowseSubdirPopupMenu;
 GtkWidget *BrowserList;
 GtkWidget *BrowserArtistList;
 GtkWidget *BrowserAlbumList;
-GtkWidget *BrowserEntry;
+GtkWidget *BrowserEntryCombo;
+GtkListStore *BrowserEntryModel;
 GtkWidget *BrowserHPaned;
 GtkWidget *ArtistAlbumVPaned;
 
@@ -90,25 +77,29 @@ gint        Browser_Tree_Select_Dir (gchar *current_path);
 void        Browser_Tree_Rebuild    (gchar *path_to_load);
 void        Browser_Tree_Collapse   (void);
 
-void        Browser_List_Load_All_Files          (void);
-void        Browser_List_Load_File_List          (GList *etfilelist);
+void        Browser_List_Load_File_List          (GList *etfilelist, ET_File *etfile_to_select);
 void        Browser_List_Refresh_Whole_List      (void);
 void        Browser_List_Refresh_File_In_List    (ET_File *ETFile);
 void        Browser_List_Clear                   (void);
-void        Browser_List_Select_File             (ET_File *ETFile, gboolean select_it);
-void        Browser_List_Select_File_By_Row      (guint row, gboolean select_it);
-void        Browser_List_Unselect_File           (ET_File *ETFile);
-void        Browser_List_Unselect_File_By_Row    (guint row);
-void        Browser_List_Scroll_Vertical         (GtkCList *clist, GtkScrollType scroll_type,
-                                                  gfloat position, gpointer user_data);
-void        Browser_List_Set_Row_Color           (guint row);
-GList      *Browser_List_Get_Selection           (void);
+void        Browser_List_Select_File_By_Etfile   (ET_File *ETFile, gboolean select_it);
+void        Browser_List_Select_File_By_Iter     (GtkTreeIter *iter, gboolean select_it);
+void        Browser_List_Select_File_By_Iter_String(const gchar* stringiter, gboolean select_it);
+ET_File    *Browser_List_Select_File_By_DLM      (const gchar* string, gboolean select_it);
+void        Browser_List_Unselect_File_By_Etfile (ET_File *ETFile);
+void        Browser_List_Unselect_File_By_Iter   (GtkTreeIter *iter);
+void        Browser_List_Unselect_File_By_Iter_String(const gchar* stringiter);
+void        Browser_List_Scroll_Vertical      (/*GtkCList *clist, GtkScrollType scroll_type,
+                                               gfloat position, gpointer user_data*/);
+void        Browser_List_Set_Row_Appearance      (GtkTreeIter *iter);
+void        Browser_List_Refresh_Sort            (void);
 void        Browser_List_Select_All_Files        (void);
 void        Browser_List_Unselect_All_Files      (void);
 void        Browser_List_Invert_File_Selection   (void);
 void        Browser_List_Remove_File             (ET_File *ETFile);
+ET_File    *Browser_List_Get_ETFile_From_Path    (GtkTreePath *path);
+ET_File    *Browser_List_Get_ETFile_From_Iter    (GtkTreeIter *iter);
 
-void        Browser_Artist_List_Load_Files       (void);
+void        Browser_Artist_List_Load_Files       (ET_File *etfile_to_select);
 
 void        Browser_Entry_Set_Text      (gchar *text);
 void        Browser_Label_Set_Text      (gchar *text);
