@@ -24,6 +24,7 @@
 
 
 #include <glib.h>
+#include <gdk/gdktypes.h>
 #include <config.h> // For definition of ENABLE_OGG
 
 
@@ -49,6 +50,7 @@ GdkColor LIGHT_RED;
 GdkColor GREY;
 GdkColor LIGHT_GREY;
 GdkColor YELLOW;
+GdkColor BLACK;
 
 
 /*
@@ -56,34 +58,44 @@ GdkColor YELLOW;
  */
 typedef enum
 {
-    SORTING_BY_ASCENDING_FILENAME       = 0,
-    SORTING_BY_DESCENDING_FILENAME      = 1,
-    SORTING_BY_ASCENDING_TRACK_NUMBER   = 3,
-    SORTING_BY_DESCENDING_TRACK_NUMBER  = 4,
-    SORTING_BY_ASCENDING_CREATION_DATE  = 6,
-    SORTING_BY_DESCENDING_CREATION_DATE = 7,
-    SORTING_BY_ASCENDING_TITLE          = 9,
-    SORTING_BY_DESCENDING_TITLE         = 10,
-    SORTING_BY_ASCENDING_ARTIST         = 12,
-    SORTING_BY_DESCENDING_ARTIST        = 13,
-    SORTING_BY_ASCENDING_ALBUM          = 15,
-    SORTING_BY_DESCENDING_ALBUM         = 16,
-    SORTING_BY_ASCENDING_YEAR           = 18,
-    SORTING_BY_DESCENDING_YEAR          = 19,
-    SORTING_BY_ASCENDING_GENRE          = 21,
-    SORTING_BY_DESCENDING_GENRE         = 22,
-    SORTING_BY_ASCENDING_COMMENT        = 24,
-    SORTING_BY_DESCENDING_COMMENT       = 25,
-    SORTING_BY_ASCENDING_FILE_TYPE      = 27,
-    SORTING_BY_DESCENDING_FILE_TYPE     = 28,
-    SORTING_BY_ASCENDING_FILE_SIZE      = 30,
-    SORTING_BY_DESCENDING_FILE_SIZE     = 31,
-    SORTING_BY_ASCENDING_FILE_DURATION  = 33,
-    SORTING_BY_DESCENDING_FILE_DURATION = 34,
-    SORTING_BY_ASCENDING_FILE_BITRATE   = 36,
-    SORTING_BY_DESCENDING_FILE_BITRATE  = 37,
-    SORTING_BY_ASCENDING_FILE_SAMPLERATE  = 39,
-    SORTING_BY_DESCENDING_FILE_SAMPLERATE = 40,
+    SORTING_BY_ASCENDING_FILENAME,
+    SORTING_BY_DESCENDING_FILENAME,
+    SORTING_BY_ASCENDING_TRACK_NUMBER,
+    SORTING_BY_DESCENDING_TRACK_NUMBER,
+    SORTING_BY_ASCENDING_CREATION_DATE,
+    SORTING_BY_DESCENDING_CREATION_DATE,
+    SORTING_BY_ASCENDING_TITLE,
+    SORTING_BY_DESCENDING_TITLE,
+    SORTING_BY_ASCENDING_ARTIST,
+    SORTING_BY_DESCENDING_ARTIST,
+    SORTING_BY_ASCENDING_ALBUM,
+    SORTING_BY_DESCENDING_ALBUM,
+    SORTING_BY_ASCENDING_YEAR,
+    SORTING_BY_DESCENDING_YEAR,
+    SORTING_BY_ASCENDING_GENRE,
+    SORTING_BY_DESCENDING_GENRE,
+    SORTING_BY_ASCENDING_COMMENT,
+    SORTING_BY_DESCENDING_COMMENT,
+    SORTING_BY_ASCENDING_COMPOSER,
+    SORTING_BY_DESCENDING_COMPOSER,
+    SORTING_BY_ASCENDING_ORIG_ARTIST,
+    SORTING_BY_DESCENDING_ORIG_ARTIST,
+    SORTING_BY_ASCENDING_COPYRIGHT,
+    SORTING_BY_DESCENDING_COPYRIGHT,
+    SORTING_BY_ASCENDING_URL,
+    SORTING_BY_DESCENDING_URL,
+    SORTING_BY_ASCENDING_ENCODED_BY,
+    SORTING_BY_DESCENDING_ENCODED_BY,
+    SORTING_BY_ASCENDING_FILE_TYPE,
+    SORTING_BY_DESCENDING_FILE_TYPE,
+    SORTING_BY_ASCENDING_FILE_SIZE,
+    SORTING_BY_DESCENDING_FILE_SIZE,
+    SORTING_BY_ASCENDING_FILE_DURATION,
+    SORTING_BY_DESCENDING_FILE_DURATION,
+    SORTING_BY_ASCENDING_FILE_BITRATE,
+    SORTING_BY_DESCENDING_FILE_BITRATE,
+    SORTING_BY_ASCENDING_FILE_SAMPLERATE,
+    SORTING_BY_DESCENDING_FILE_SAMPLERATE,
     SORTING_UNKNOWN
 } ET_Sorting_Type;
 
@@ -131,6 +143,22 @@ struct _File_Name
 };
 
 
+/*
+ * Description of Picture item (see picture.h)
+ */
+typedef struct _Picture Picture;
+struct _Picture
+{
+    gint     type;
+    gchar   *description;
+    gint     width;         /* Original width of the picture */
+    gint     height;        /* Original height of the picture */
+    gulong   size;          /* Picture size in bits */
+    guchar  *data;
+    Picture *next;
+};
+
+
 /*
  * Description of each item of the TagList list
  */
@@ -139,6 +167,7 @@ struct _File_Tag
 {
     guint key;             /* Incremented value */
     gboolean saved;        /* Set to TRUE if this tag had been saved */
+
     gchar *title;          /* Title of track */
     gchar *artist;         /* Artist name */
     gchar *album;          /* Album name */
@@ -147,6 +176,12 @@ struct _File_Tag
     gchar *track_total;    /* The number of tracks for the album (ex: 12/20) */
     gchar *genre;          /* Genre of song */
     gchar *comment;        /* Comment */
+    gchar *composer;       /* Composer */
+    gchar *orig_artist;    /* Original artist / Performer */
+    gchar *copyright;      /* Copyright */
+    gchar *url;            /* URL */
+    gchar *encoded_by;     /* Encoded by */
+    Picture *picture;      /* Picture */
     GList *other;          /* List of unsupported fields (used for ogg only) */
 };
 
@@ -158,17 +193,17 @@ struct _File_Tag
 typedef struct _ET_File_Info ET_File_Info;
 struct _ET_File_Info
 {
-    gint version;          /* Version of bitstream (mpeg version for mp3, encoder version for ogg) */
-    gint mpeg25;           /* Version is MPEG 2.5? */
-    gint layer;            /* "MP3 data" */
-    gint bitrate;          /* Bitrate (kb/s) */
-    gboolean variable_bitrate; /* Is a VBR file? */
-    gint samplerate;       /* Samplerate (Hz) */
-    gint mode;             /* Stereo, ... or channels for ogg */
-    gint size;             /* The size of file (in bytes) */
-    gint duration;         /* The duration of file (in seconds) */
-    gchar *mpc_profile;    /* MPC data */
-    gchar *mpc_version;    /* MPC data : encoder version */
+    gint version;               /* Version of bitstream (mpeg version for mp3, encoder version for ogg) */
+    gint mpeg25;                /* Version is MPEG 2.5? */
+    gint layer;                 /* "MP3 data" */
+    gint bitrate;               /* Bitrate (kb/s) */
+    gboolean variable_bitrate;  /* Is a VBR file? */
+    gint samplerate;            /* Samplerate (Hz) */
+    gint mode;                  /* Stereo, ... or channels for ogg */
+    gint size;                  /* The size of file (in bytes) */
+    gint duration;              /* The duration of file (in seconds) */
+    gchar *mpc_profile;         /* MPC data */
+    gchar *mpc_version;         /* MPC data : encoder version */
 };
 
 
@@ -187,7 +222,7 @@ struct _ET_File_Description
 /*
  * Description of supported files
  */
-static ET_File_Description ETFileDescription[] = 
+static const ET_File_Description ETFileDescription[] = 
 {
     {MP3_FILE,     ".mp3",  ID3_TAG},
 #ifdef ENABLE_OGG
@@ -210,6 +245,7 @@ static ET_File_Description ETFileDescription[] =
     {OFR_FILE,     ".ofs",  APE_TAG},
     {UNKNOWN_FILE, "",      UNKNOWN_TAG}    /* This item must be placed to the end! */
 };
+
 /* Calculate the last index of the previous tab */
 #define ET_FILE_DESCRIPTION_SIZE ( sizeof(ETFileDescription)/sizeof(ETFileDescription[0]) - 1 )
 
@@ -225,7 +261,8 @@ struct _ET_File
     guint ETFileKey;          /* Primary key to identify each item of the list (no longer used?) */
 
     ET_File_Description *ETFileDescription;
-    ET_File_Info        *ETFileInfo; /* Header infos: bitrate, duration, ... */
+    gchar               *ETFileExtension;   /* Real extension of the file (keeping the case) (should be placed in ETFileDescription?) */
+    ET_File_Info        *ETFileInfo;        /* Header infos: bitrate, duration, ... */
 
     GList *FileNameCur;       /* Points to item of FileNameList that represents the current value of filename state (i.e. file on hard disk) */
     GList *FileNameNew;       /* Points to item of FileNameList that represents the new value of filename state */
@@ -275,11 +312,6 @@ struct _ET_Core
     ET_File *ETFileDisplayed;           // Pointer to the current ETFile displayed in EasyTAG (may be NULL)
 
 
-    // Selection list
-    GList *ETFileSelectionList;         // List of files selected (in BrowserList) for an action (Taken in ETFileList) (List of ET_File)
-    guint  ETFileSelectionList_Length;  // Contains the length of the list
-
-
     // History list
     GList *ETHistoryFileList;           // History list of files changes for undo/redo actions
 };
@@ -310,15 +342,17 @@ gboolean ET_Check_If_All_Files_Are_Saved (void);
 
 //gboolean ET_Free_File_List               (void);
 gboolean ET_Free_History_File_List       (void);
-gboolean ET_Free_Selection_File_List     (void);
 gboolean ET_Free_Displayed_File_List     (void);
 
-File_Name *ET_File_Name_Item_New (void);
-File_Tag  *ET_File_Tag_Item_New  (void);
+ET_File   *ET_File_Item_New       (void);
+File_Name *ET_File_Name_Item_New  (void);
+File_Tag  *ET_File_Tag_Item_New   (void);
+gboolean   ET_Free_File_List_Item (ET_File *ETFile);
 
-gboolean ET_Copy_File_Tag_Item       (ET_File *ETFile, File_Tag *FileTag);
-gboolean ET_Set_Field_File_Name_Item (gchar **FileNameField, gchar *value);
-gboolean ET_Set_Field_File_Tag_Item  (gchar **FileTagField,  gchar *value);
+gboolean ET_Copy_File_Tag_Item         (ET_File *ETFile, File_Tag *FileTag);
+gboolean ET_Set_Field_File_Name_Item   (gchar **FileNameField, gchar *value);
+gboolean ET_Set_Field_File_Tag_Item    (gchar **FileTagField,  gchar *value);
+gboolean ET_Set_Field_File_Tag_Picture (gchar **FileTagField, Picture *pic);
 
 GList   *ET_Displayed_File_List_First       (void);
 GList   *ET_Displayed_File_List_Previous    (void);
@@ -331,8 +365,6 @@ guint    ET_Displayed_File_List_Get_Length  (void);
 
 gboolean ET_Set_Displayed_File_List         (GList *ETFileList);
 
-GList   *ET_File_List_Get_Selection         (void);
-
 void     ET_Display_File_Data_To_UI (ET_File *ETFile);
 void     ET_Save_File_Data_From_UI  (ET_File *ETFile);
 gboolean ET_Save_File_Tag_To_HD     (ET_File *ETFile);
@@ -351,8 +383,9 @@ gboolean ET_Manage_Changes_Of_File_Data          (ET_File *ETFile, File_Name *Fi
 void     ET_Mark_File_Name_As_Saved              (ET_File *ETFile);
 void     ET_Update_Directory_Name_Into_File_List (gchar* last_path, gchar *new_path);
 gboolean ET_File_Name_Convert_Character          (gchar *filename);
+gchar   *ET_File_Name_Format_Extension           (ET_File *ETFile);
+gchar   *ET_File_Name_Generate                   (ET_File *ETFile, gchar *new_file_name);
 guint    ET_Get_Number_Of_Files_In_Directory     (gchar *path);
-gchar   *ET_Generate_File_Name                   (ET_File *ETFile, gchar *new_file_name);
 
 void ET_Sort_Displayed_File_List                          (ET_Sorting_Type Sorting_Type);
 void ET_Sort_Displayed_File_List_And_Update_UI            (ET_Sorting_Type Sorting_Type);
@@ -374,6 +407,16 @@ gint ET_Comp_Func_Sort_File_By_Ascending_Genre            (ET_File *ETFile1, ET_
 gint ET_Comp_Func_Sort_File_By_Descending_Genre           (ET_File *ETFile1, ET_File *ETFile2);
 gint ET_Comp_Func_Sort_File_By_Ascending_Comment          (ET_File *ETFile1, ET_File *ETFile2);
 gint ET_Comp_Func_Sort_File_By_Descending_Comment         (ET_File *ETFile1, ET_File *ETFile2);
+gint ET_Comp_Func_Sort_File_By_Ascending_Composer         (ET_File *ETFile1, ET_File *ETFile2);
+gint ET_Comp_Func_Sort_File_By_Descending_Composer        (ET_File *ETFile1, ET_File *ETFile2);
+gint ET_Comp_Func_Sort_File_By_Ascending_Orig_Artist      (ET_File *ETFile1, ET_File *ETFile2);
+gint ET_Comp_Func_Sort_File_By_Descending_Orig_Artist     (ET_File *ETFile1, ET_File *ETFile2);
+gint ET_Comp_Func_Sort_File_By_Ascending_Copyright        (ET_File *ETFile1, ET_File *ETFile2);
+gint ET_Comp_Func_Sort_File_By_Descending_Copyright       (ET_File *ETFile1, ET_File *ETFile2);
+gint ET_Comp_Func_Sort_File_By_Ascending_Url              (ET_File *ETFile1, ET_File *ETFile2);
+gint ET_Comp_Func_Sort_File_By_Descending_Url             (ET_File *ETFile1, ET_File *ETFile2);
+gint ET_Comp_Func_Sort_File_By_Ascending_Encoded_By       (ET_File *ETFile1, ET_File *ETFile2);
+gint ET_Comp_Func_Sort_File_By_Descending_Encoded_By      (ET_File *ETFile1, ET_File *ETFile2);
 
 gint ET_Comp_Func_Sort_File_By_Ascending_File_Type        (ET_File *ETFile1, ET_File *ETFile2);
 gint ET_Comp_Func_Sort_File_By_Descending_File_Type       (ET_File *ETFile1, ET_File *ETFile2);
