@@ -1,7 +1,7 @@
 /* ogg_tag.h - 2001/11/08 */
 /*
- *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2001-2002  Jerome Couderc <j.couderc@ifrance.com>
+ *  EasyTAG - Tag editor for MP3 and Ogg Vorbis files
+ *  Copyright (C) 2001-2003  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,8 +39,4 @@ gboolean Ogg_Tag_Read_File_Tag  (gchar *filename, File_Tag *FileTag);
 gboolean Ogg_Tag_Write_File_Tag (ET_File *ETFile);
 
 
-gboolean Ogg_Header_Read_File_Info          (gchar *filename, ET_File_Info *ETFileInfo);
-gboolean Ogg_Header_Display_File_Info_To_UI (gchar *filename, ET_File_Info *ETFileInfo);
-
-
 #endif /* __OGG_TAG_H__ */
