@@ -40,14 +40,17 @@ GtkWidget *BrowserLineStyleOptionMenu;
 GtkWidget *BrowserExpanderStyleOptionMenu;
 GtkWidget *WriteId3v1Tag;
 GtkWidget *WriteId3v2Tag;
+GtkWidget *WriteId3TagsInFlacFiles;
 GtkWidget *UseId3v2Padding;
 GtkWidget *StripTagWhenEmptyFields;
 GtkWidget *BrowseSubdir;
+GtkWidget *BrowseHiddendir;
 GtkWidget *OpenSelectedBrowserNode;
-GtkWidget *KeepTreeBrowserInMemory;
 GtkWidget *ConvertOldId3v2TagVersion;
 GtkWidget *SaveConfigBeforeExit;
 
+GtkListStore *DefaultPathModel;
+
 /* User interface */
 GtkWidget *SetMainWindowSize;
 GtkWidget *MainWindowHeight;
@@ -65,31 +68,37 @@ GtkWidget *PaneHandleButton3;
 
 GtkWidget *ShowHeaderInfos;
 GtkWidget *UseColoredTooltips;
+GtkWidget *UseNewListStyle;
 
 /* Misc */
 GtkWidget *ReplaceIllegalCharactersInFilename;
+GtkWidget *FilenameExtensionNoChange;
+GtkWidget *FilenameExtensionLowerCase;
+GtkWidget *FilenameExtensionUpperCase;
 GtkWidget *DateAutoCompletion;
 GtkWidget *NumberTrackFormated;
 GtkWidget *NumberTrackFormatedSpinButton;
 GtkWidget *OggTagWriteXmmsComment;
-GtkWidget *SortingFileOptionMenu;
+GtkWidget *SortingFileCombo;
 GtkWidget *SortingFileCaseSensitive;
 GtkWidget *UseISO88591CharacterSetTranslation;
 GtkWidget *UseCharacterSetTranslation;
-GtkWidget *FileCharacterSetEntry;
-GtkWidget *UserCharacterSetEntry;
+GtkWidget *FileCharacterSetCombo;
+GtkWidget *UserCharacterSetCombo;
 
 GtkWidget *MessageBoxPositionNone;
 GtkWidget *MessageBoxPositionCenter;
 GtkWidget *MessageBoxPositionMouse;
 
-GtkWidget *FilePlayerEntry;
+GtkWidget *FilePlayerCombo;
+GtkListStore *FilePlayerModel;
 
 /* Scanner */
 GtkWidget *FTSConvertUnderscoreAndP20IntoSpace;
 GtkWidget *FTSConvertSpaceIntoUnderscore;
 GtkWidget *RFSConvertUnderscoreAndP20IntoSpace;
 GtkWidget *RFSConvertSpaceIntoUnderscore;
+GtkWidget *PFSDontUpperSomeWords;
 GtkWidget *OverwriteTagField;
 GtkWidget *OpenScannerWindowOnStartup;
 GtkWidget *ScannerWindowOnTop;
@@ -97,6 +106,7 @@ GtkWidget *ScannerWindowOnTop;
 GtkWidget *SetDefaultComment;
 GtkWidget *DefaultComment;
 GtkWidget *Crc32Comment;
+GtkListStore *DefaultCommentModel;
 
 GtkWidget *SetScannerWindowPosition;
 GtkWidget *ScannerWindowX;
@@ -121,6 +131,8 @@ GtkWidget *SetCddbPaneHandlePosition;
 GtkWidget *CddbPaneHandlePosition;
 GtkWidget *CddbPaneHandleButton;
 
+GtkWidget *CddbFollowFile;
+
 
 /* Confirmation */
 GtkWidget *ConfirmBeforeExit;
