@@ -13,7 +13,7 @@
 
 #ifdef ENABLE_OGG
 
-
+#include <glib/gi18n-lib.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -22,7 +22,6 @@
 #include <vorbis/codec.h>
 
 #include "vcedit.h"
-#include "i18n.h"
 
 
 #define CHUNKSIZE 4096
